﻿using System;
using HEAL.Attic;
using HeuristicLab.Common;
using HeuristicLab.Encodings.RealVectorEncoding;

namespace HeuristicLab.Algorithms.NSGA3
{
    [StorableType("D35C9D79-77DF-4CF3-AB44-FCCC00E44C5F")]
    public class Solution : IDeepCloneable
    {
        // Chromosome
        [Storable]
        public RealVector Chromosome { get; set; }

        // actual fitness of solution as given by Problem
        [Storable]
        public double[] Objectives { get; set; }

        // normalized fitness used in selection process (in order to not overwrite the original Fitness)
        [Storable]
        public double[] NormalizedObjectives { get; set; }

        [StorableConstructor]
        public Solution(StorableConstructorFlag _) { }

        public Solution(RealVector chromosome)
        {
            Chromosome = chromosome;
        }

        public Solution(Solution solution, Cloner cloner)
        {
            Chromosome = cloner.Clone(solution.Chromosome);
            if (solution.Objectives != null)
            {
                Objectives = new double[solution.Objectives.Length];
                Array.Copy(solution.Objectives, Objectives, solution.Objectives.Length);
            }
            if (solution.NormalizedObjectives != null)
            {
                NormalizedObjectives = new double[solution.NormalizedObjectives.Length];
                Array.Copy(solution.NormalizedObjectives, NormalizedObjectives, solution.NormalizedObjectives.Length);
            }
        }

        public IDeepCloneable Clone(Cloner cloner)
        {
            return new Solution(this, cloner);
        }

        public object Clone()
        {
            return new Cloner().Clone(this);
        }
    }
}